package messagequeue

import (
	"query-service/internal/errorhandler"

	"github.com/streadway/amqp"
	"github.com/thijsheijden/alice"
)

// ConsumeMessageFunc is a function type to be called when a message is consumed
type ConsumeMessageFunc func(amqp.Delivery)

// StartConsumer will start a consumer
// When a message is received the consumeMessage function will be called
func StartConsumer(broker *alice.RabbitBroker, exchangeID *string, routingKey *string, consumeMessage ConsumeMessageFunc) {
	// Declare the exchange we want to bind to
	exchange, err := alice.CreateDefaultExchange(*exchangeID, alice.Direct)
	if err != nil {
		errorhandler.FailWithError(err, "failed to create exchange")
	}

	// Declare the queue we will consume from
	queue := alice.CreateQueue(exchange, "aql-query-queue", true, false, true, false, nil)

	// Create the consumer
	c, err := broker.CreateConsumer(queue, *routingKey, alice.DefaultConsumerErrorHandler)
	if err != nil {
		errorhandler.FailWithError(err, "failed to create consumer")
	}

	// Start consuming messages
	go c.ConsumeMessages(nil, false, func(msg amqp.Delivery) { consumeMessage(msg) })
}
