package request

import (
	"context"
	"crypto/tls"
	"fmt"
	"log"

	"encoding/json"
	"io/ioutil"

	driver "github.com/arangodb/go-driver"
	"github.com/arangodb/go-driver/http"
)

// Document with Empty struct to retrieve all data from the DB Document
type Document map[string]interface{}

// GeneralFormat with Empty struct to retrieve all data from the DB Document
type GeneralFormat map[string][]Document

//attr interface{}

//map[1 , 2 , 3 map [ .. ]]

// SendAQLQuery send AQL string query to database and returns a JSON object in a general format
func SendAQLQuery(AQLQuery string) (*[]byte, error) {
	var queryResult []Document
	conn, err := http.NewConnection(http.ConnectionConfig{
		Endpoints: []string{"https://aae8f5c054da.arangodb.cloud:8529"},
		TLSConfig: &tls.Config{InsecureSkipVerify: true},
	})
	if err != nil {
		log.Println("could not connect to database") // Handle error
		return nil, err
	}
	c, err := driver.NewClient(driver.ClientConfig{
		Connection:     conn,
		Authentication: driver.BasicAuthentication("root", "kXR0PFepIVm3EqgQ2MVc"),
	})
	if err != nil {
		log.Println("Could not log in to the ArangoDB") // Handle error
		return nil, err
	}

	ctx := context.Background()
	db, err := c.Database(ctx, "_system")
	if err != nil {
		// handle error
		log.Println(err)
		return nil, err
	}

	// CHANGED TO OTHER FORMAT
	//fmt.Println(AQLQuery)
	// query := `
	// 			LET n0 = (
	// 				FOR x IN airports
	// 				FILTER  x.country == 'USA'
	// 				RETURN x
	// 			)
	// 			FOR n in n0
	// 			RETURN n
	// 		`
	cursor, err := db.Query(ctx, AQLQuery, nil)
	if err != nil {
		log.Println("Invalid query") // handle error
		return nil, err
	}
	defer cursor.Close()
	for {
		var doc GeneralFormat
		_, err := cursor.ReadDocument(ctx, &doc)
		if driver.IsNoMoreDocuments(err) {
			break
		} else if err != nil {
			// handle other errors
			return nil, err
		}
		// fmt.Printf("%T\n", doc)
		queryResult = append(queryResult, formatToJSON(doc)...)
		//fmt.Println(doc)
		//formatToJSON(doc)
	}

	writeJSON(queryResult)
	file, err := json.MarshalIndent(queryResult, "", " ")
	return &file, err
}

func formatToJSON(doc GeneralFormat) []Document {
	//b, err := json.Marshal(doc)
	//if err != nil {
	//handle error
	//}
	// fmt.Println(doc)

	var nodeList []Document
	for _, v := range doc {
		for _, j := range v {
			nodeList = append(nodeList, parseData(j))
		}
	}
	fmt.Println(nodeList)
	return nodeList
}

func writeJSON(queryResult []Document) {
	file, _ := json.MarshalIndent(queryResult, "", " ")

	_ = ioutil.WriteFile("result.json", file, 0644)
}

func parseData(doc Document) Document {
	data := make(Document)
	data["_id"] = doc["_id"]
	delete(doc, "_id")
	data["_key"] = doc["_key"]
	delete(doc, "_key")
	data["_rev"] = doc["_rev"]
	delete(doc, "_rev")
	data["attributes"] = doc

	//delete(doc, "_key")
	//data.rev = fmt.Sprintf("%v", doc["_rev"])

	// delete(doc, "_rev")
	// data.attr = doc
	return data
}
